@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -144,7 +144,7 @@ class _UserBase(CollectionMixin):
 
     def _get_collection(self, request):
         """See `CollectionMixin`."""
-        return list(getUtility(IUserManager).users)
+        return getUtility(IUserManager).users
 
 
 @public
@@ -163,9 +163,45 @@ class AllUsers(_UserBase):
             arguments = validator(request)
             create_user(self.api, arguments, response)
         except (ValueError, InvalidEmailAddressError) as error:
+            error_message = 'Invalid email address {}'.format(error)
+            bad_request(response, error_message)
+            return
+
+
+class _FoundUsers(_UserBase):
+    def __init__(self, users, api):
+        super().__init__()
+        self._users = users
+        self.api = api
+
+    def _get_collection(self, request):
+        return self._users
+
+
+class FindUsers(_UserBase):
+
+    def on_get(self, request, response):
+        validator = Validator(q=str,
+                              # Allow pagination.
+                              page=int,
+                              count=int,
+                              _optional=('page', 'count'))
+        try:
+            data = validator(request)
+        except ValueError as error:
             bad_request(response, str(error))
             return
 
+        user_manager = getUtility(IUserManager)
+        users = list(user_manager.find_users(data.get('q')))
+        resource = _FoundUsers(users, self.api)
+        try:
+            collection = resource._make_collection(request)
+        except ValueError as ex:
+            bad_request(response, str(ex))
+        else:
+            okay(response, etag(collection))
+
 
 @public
 class AUser(_UserBase):
